/*
 *  File: wav.h           Created 05-DEC-2001        Richard B. Johnson
 *                                                   rjohnson@analogic.com
 *  Released under GPL.
 */

#ifndef _WAV_H_
#define _WAV_H_

#define	WAV_FMT_MICROSLUT         0x0000
#define	WAV_FMT_PCM               0x0001
#define	WAV_FMT_ADPCM             0x0002
#define WAV_FMT_IEEE_FLOAT        0x0003
#define	WAV_FMT_ALAW              0x0006
#define	WAV_FMT_MULAW             0x0007
#define	WAV_FMT_OKI_ADPCM         0x0010
#define WAV_FMT_IMA_ADPCM         0x0011
#define	WAV_FMT_DIGISTD           0x0015
#define	WAV_FMT_DIGIFIX           0x0016
#define WAV_FMT_DOLBY_AC2         0x0030
#define WAV_FMT_GSM610            0x0031
#define WAV_FMT_ROCKWELL_ADPCM    0x003b
#define WAV_FMT_ROCKWELL_DIGITALK 0x003c
#define WAV_FMT_G721_ADPCM        0x0040
#define WAV_FMT_G728_CELP         0x0041
#define WAV_FMT_MPEG              0x0050
#define WAV_FMT_MPEGLAYER3        0x0055
#define WAV_FMT_G726_ADPCM        0x0064
#define WAV_FMT_G722_ADPCM        0x0065

#ifdef __GNUC__
#define PACKED __attribute__ ((packed))
#else
#define PACKED
#endif

#undef  offsetof
#define offsetof(t, m) ((size_t) &((t *)0)->m)
#define CHAR_BITS 0x08
#define MIN(a, b) ((unsigned)(a) < (unsigned)(b) ? (a) : (b)) 

//
//  Header of a '*.wav' file. This is in little-endian format
//  on the media.
//
typedef struct {
    int8_t  rif[4];          /* Spells 'RIFF'               */
    uint32_t len;            /* length of dat_len minus 36  */
    int8_t  wav[8];          /* Eight-byte signature        */
    uint32_t fmt_len;        /* Length of format            */
    uint16_t fmt_tag;        /* Identifies PCM, ULAW, etc.  */
    uint16_t chans;          /* Two for stereo              */
    uint32_t samp_rate;      /* Usually 44,100 Hz           */
    uint32_t bytes_sec;      /* block_size * samp_rate      */
    uint16_t block;          /* Basic block size            */
    uint16_t bps;            /* Bits per sample             */
    int8_t  dat[4];          /* Four-byte signature         */
    uint32_t dat_len;        /* Length of data that follows */
    } PACKED WAV;

//
//  The header files __should__ make these macros work for both
//  little and big endian. The PDP-11/34 endian is not handled.
//

#define WORD_SWAP(a)  ( (((uint32_t)(a) & 0x000000ffU) << 0x18 ) | \
                        (((uint32_t)(a) & 0x0000ff00U) << 0x08 ) | \
                        (((uint32_t)(a) & 0x00ff0000U) >> 0x08 ) | \
                        (((uint32_t)(a) & 0xff000000U) >> 0x18 ) )

#define BYTE_SWAP(a) ( (((uint16_t)(a) & 0x00ffU) << 0x08) | \
                       (((uint16_t)(a) & 0xff00U) >> 0x08) )


#if BYTE_ORDER == LITTLE_ENDIAN
#define U16(i) ((uint16_t)(i))
#define U32(i) ((uint32_t)(i)) 
#define I16(i) ((int16_t)(i))
#elif BYTE_ORDER == BIG_ENDIAN
#define U16(i) ((uint16_t)BYTE_SWAP(i))
#define U32(i) ((uint32_t)WORD_SWAP(i)) 
#define I16(i) ((int16_t)BYTE_SWAP(i))
#else
#error Byte-order not defined
#endif
#endif

